/*!
	\file Food.cpp

	\brief cpp-Datei der Klasse CFood
	\author Christian Roesch
	\date 21.6.2010

	Erklaerung	
*/

#include "Food.h"
#include "helper.h"
#include "globals.h"
#include "Player.h"

#include <cmath>
#include "resources.h"
#include "Game.h"


/*!
	Der Konstruktor
*/
Food::Food()
{
	Reset();
}

void Food::Reset()
{
	m_fPosX= 0.0f;
	m_fPosY= 0.0f;
	m_fSpeedX= 0.0f;
	m_fSpeedY= 0.0f;
}

void Food::SetPos( float fPosX, float fPosY )
{
	m_fPosX= fPosX;
	m_fPosY= fPosY;
}

float Food::GetX()
{
	return m_fPosX;
}

float Food::GetY()
{
	return m_fPosY;
}

float Food::GetNextX()
{
	return m_fPosX + m_fSpeedX * 0.0625f;
}

float Food::GetNextY()
{
	return m_fPosY + m_fSpeedY * 0.0625f;
}

void Food::draw()
{
	sprFood->RenderEx( 
		m_fPosX + g_BoardSizeX, 
		m_fPosY + g_BoardSizeY,
		0.0f,
		g_FoodSize / 2.0f);
}

void Food::calculate( float fTimeStep, std::vector< Food >& allFood, Player* p )
{
	float fMaxDist= 30.0f;

	float fNearest= fMaxDist * fMaxDist;
	float fNearestX= fMaxDist;
	float fNearestY= fMaxDist;

	float fCenterX= 0.0f;
	float fCenterY= 0.0f;
	int NearCount= 0;

	for( int i= 0; i < allFood.size(); ++i )
	{
		float fDistX= allFood[ i ].GetX() - GetX();
		float fDistY= allFood[ i ].GetY() - GetY();
		float fDist= fDistX * fDistX + fDistY * fDistY;
		if( fDist <= fMaxDist * fMaxDist )
		{
			if( &(allFood[ i ]) == this )
			{
				continue;
			}
			fCenterX+= allFood[ i ].GetNextX();
			fCenterY+= allFood[ i ].GetNextY();
			NearCount++;

			if( fNearest > fDist )
			{
				fNearest= fDist;
				fNearestX= allFood[ i ].GetX();
				fNearestY= allFood[ i ].GetY();
			}
		}
	}

	float fAcc= 200.0f * fTimeStep;
	float fAccX= 0.0f;
	float fAccY= 0.0f;

	if( NearCount > 0 )
	{
		fCenterX/= (float)NearCount;
		fCenterY/= (float)NearCount;
		fCenterX-= m_fPosX;
		fCenterY-= m_fPosY;

		fAccX+= fCenterX * 0.125f;
		fAccY+= fCenterY * 0.125f;
	}

	{
		fNearestX= m_fPosX - fNearestX;
		fNearestY= m_fPosY - fNearestY;
		float fFac= 1.0f - sqrt( fNearest ) / fMaxDist;
		fAccX+= fNearestX / sqrt( fNearest ) * fFac * 3.75f;
		fAccY+= fNearestY / sqrt( fNearest ) * fFac * 3.75f;
	}

	for( int j= 0; j < g_Game.GetPlayerCount(); ++j )
	{
		float fMaxSteerDist= p[ j ].GetFoodPower();
		float fSteerX= p[ j ].GetX() - m_fPosX;
		float fSteerY= p[ j ].GetY() - m_fPosY;
		float fSteerDist= sqrt( fSteerX * fSteerX + fSteerY * fSteerY );
		if( fSteerDist < g_GhostSize * 2.5f )
		{
			p[ j ].FoodTouch( fTimeStep );
		}
		if( fSteerDist < fMaxSteerDist )
		{
			float fFac= 1.0f - fSteerDist / fMaxSteerDist;
			fAccX+= fSteerX / fSteerDist * fFac;
			fAccY+= fSteerY / fSteerDist * fFac;
		}
	}

	{
		float fBorderSpeed= 16.0f;
		float fBorderDist= 8.0f * g_FoodSize;

		if( GetNextX() > g_BoardSizeX - fBorderDist )
		{
			fAccX-= fBorderSpeed;
		}
		if( GetNextX() < -g_BoardSizeX + fBorderDist )
		{
			fAccX+= fBorderSpeed;
		}
		if( GetNextY() > g_BoardSizeY - fBorderDist )
		{
			fAccY-= fBorderSpeed;
		}
		if( GetNextY() < -g_BoardSizeY + fBorderDist )
		{
			fAccY+= fBorderSpeed;
		}
	}

	{
		float fLength= sqrt( fAccX * fAccX + fAccY * fAccY );
		if( fLength > 0.05f )
		{
			fLength= max( 1.0f, fLength );
			m_fSpeedX+= fAcc * fAccX / fLength;
			m_fSpeedY+= fAcc * fAccY / fLength;
		}
	}


	float MaxSpeed= 100.0f;
	float fFriction= pow( 48.0f / 64.0f, fTimeStep );

	m_fSpeedX*= fFriction;
	m_fSpeedY*= fFriction;

	float fSpeed= sqrt( m_fSpeedX * m_fSpeedX + m_fSpeedY * m_fSpeedY );
	if( fSpeed > MaxSpeed )
	{
		m_fSpeedX*= MaxSpeed / fSpeed;
		m_fSpeedY*= MaxSpeed / fSpeed;
	}

	m_fPosX+= m_fSpeedX * fTimeStep;
	m_fPosY+= m_fSpeedY * fTimeStep;

	Clamp( m_fPosX, -g_BoardSizeX + g_FoodSize, g_BoardSizeX - g_FoodSize );
	Clamp( m_fPosY, -g_BoardSizeY + g_FoodSize, g_BoardSizeY - g_FoodSize );
}


//	EOF: Food.cpp
