/*!
	\file Game.cpp

	\brief cpp-Datei der Klasse CGame
	\author Christian Roesch
	\date 21.6.2010

	Erklaerung	
*/

#include "Game.h"
#include "globals.h"

#include "player.h"
#include "PacMan.h"
#include "Food.h"
#include "helper.h"
#include "resources.h"

/*!
	Der Konstruktor
*/
Game::Game()
{
	m_iPlayerCount= 1;
	m_Player= new Player[ 2 ];
}

/*!
	Der Destruktor
*/
Game::~Game()
{
	delete[] m_Player;
}

float Game::GetExtraCloneOffset()
{
	return (float)m_PacMan.size() * 1.0f + m_fLevelTime / 5.0f;
}

void Game::ResetGame()
{
	m_Player[ 0 ].Reset();
	m_Player[ 1 ].Reset();
	m_PacMan.resize( 0 );
	m_Food.resize( 0 );
	m_Level= 0;

	m_GameOver= false;
	m_fGameTime= 0.0f;
	m_fLevelTime= 0.0f;
	m_iScore= 0;
}

void Game::PrepareGameWorld()
{
	if( m_iPlayerCount == 1 )
	{
		m_Player[ 0 ].SetPos( 0.0f, -g_BoardSizeY / 2.0f );
		m_Player[ 0 ].SetKeys( HGEK_SHIFT, HGEK_UP, HGEK_LEFT, HGEK_DOWN, HGEK_RIGHT );
	}
	else
	{
		m_Player[ 0 ].SetPos( 200.0f , -g_BoardSizeY / 2.0f );	
		m_Player[ 1 ].SetPos( -200.0f, -g_BoardSizeY / 2.0f );

		m_Player[ 0 ].SetKeys( HGEK_NUMPAD0, HGEK_UP, HGEK_LEFT, HGEK_DOWN, HGEK_RIGHT );
		m_Player[ 1 ].SetKeys( HGEK_G, HGEK_W, HGEK_A, HGEK_S, HGEK_D );
	}
	int iPacManCount= GetLevel();
	float fDist= g_BoardSizeX * 0.8f;
	if( iPacManCount > 0 )
	{
		fDist/= (float)iPacManCount;
	}
	for( int i= -iPacManCount; i <= iPacManCount; ++i )
	{
		PacMan p;
		p.SetPos( (float)(i) * fDist,
			g_BoardSizeY / 2.0f );
		m_PacMan.push_back( p );
	}
}


void Game::PrepareGame()
{
	PrepareGameWorld();

	m_GameOver= false;
	m_fGameTime= 0.0f;
	m_fLevelTime= 0.0f;
	m_iScore= 0;
	
	m_Level= 0;
}

void Game::NextLevel()
{
	m_Level++;
	m_fLevelTime= 0.0f;

	m_iScore+= m_Level * 10000;

	m_PacMan.resize( 0 );
	m_Food.resize( 0 );
	PrepareGameWorld();
}


void Game::PlayerPacManCollisions()
{
	float fSize= g_GhostSize + g_PacSize;
	for( int j= 0; j < m_iPlayerCount; ++j )
	{
		for( int i= m_PacMan.size() - 1 ; i >= 0; --i )
		{
			float fDistX= m_PacMan[ i ].GetX() - m_Player[ j ].GetX();
			float fDistY= m_PacMan[ i ].GetY() - m_Player[ j ].GetY();
			if( fDistX * fDistX + fDistY * fDistY < fSize * fSize )
			{
				if( m_Player[ j ].IsVulnerable() )
				{
					//tot -> game over
					m_GameOver= true;
					playFX( sndDead, m_PacMan[ i ].GetX() );
				}
				else
				{
					//fressen
					playFX( sndPacEaten, m_PacMan[ i ].GetX() );
					m_PacMan.erase( m_PacMan.begin() + i );
					m_Player[ j ].EatPacMan();
					m_iScore+= 100 + GetLevel() * 100;
				}
			}
		}
	}

	if( m_PacMan.size() == 0 )
	{
		// -> so, jetzt neuer Level
		playFX( sndNextLevel, 0 );
		NextLevel();
	}
}

void Game::PacManFoodCollisions()
{
	float fSize= g_FoodSize + g_PacSize;
	for( int i= m_PacMan.size() - 1 ; i >= 0; --i )
	{
		for( int j= m_Food.size() - 1 ; j >= 0; --j )
		{
			float fDistX= m_PacMan[ i ].GetX() - m_Food[ j ].GetX();
			float fDistY= m_PacMan[ i ].GetY() - m_Food[ j ].GetY();
			if( fDistX * fDistX + fDistY * fDistY < fSize * fSize )
			{
				{
					//fressen
					playFX( sndFoodEaten, m_PacMan[ i ].GetX() );
					m_Food.erase( m_Food.begin() + j );
					m_PacMan[ i ].eat();
					if( m_iScore > 0 )
					{
						m_iScore-= 1;
					}
				}
			}
		}
	}
}

bool Game::FrameFunc()
{
	float dt=g_hge->Timer_GetDelta();

	// Process keys
	if (g_hge->Input_GetKeyState(HGEK_ESCAPE)) return true;


	std::vector< PacMan > NewPacMan;
	for( int i= 0; i < m_PacMan.size(); ++i )
	{
		m_PacMan[ i ].calculate( dt );
		m_PacMan[ i ].canclone( NewPacMan );
	}

	if (g_hge->Input_GetKeyState(HGEK_F1) )
	{
		m_iPlayerCount= 1;
		ResetGame();
		PrepareGame();
	}
	if (g_hge->Input_GetKeyState(HGEK_F2) )
	{
		m_iPlayerCount= 2;
		ResetGame();
		PrepareGame();
	}
	if( m_GameOver )
	{
		if (g_hge->Input_GetKeyState(HGEK_R) || g_hge->Input_GetKeyState(HGEK_SPACE))
		{
			ResetGame();
			PrepareGame();
		}
	}
	else
	{
		m_fGameTime+= dt;
		m_fLevelTime+= dt;
		m_PacMan.insert( m_PacMan.end(), NewPacMan.begin(), NewPacMan.end() );
		for( int i= 0; i < m_iPlayerCount; ++i )
		{
			m_Player[ i ].calculate( dt );
		}

		PlayerPacManCollisions();
		PacManFoodCollisions();

		for( int i= 0; i < m_Food.size(); ++i )
		{
			m_Food[ i ].calculate( dt, m_Food, m_Player );
		}
	}

	//eigentlich Test
	if( g_hge->Random_Float(0.0f, 1.0f) > 0.9f + (float)m_Food.size() * 0.0002f )
	{
		Food f;
		f.SetPos( 
			g_hge->Random_Float(-400.0f, 400.0f),
			g_hge->Random_Float(-300.0f, 300.0f) );

		m_Food.push_back( f );
	}

	return false;
}

bool Game::RenderFunc()
{
	// Render graphics
	g_hge->Gfx_BeginScene();
	g_hge->Gfx_Clear(0);
	for( int i= 0; i < m_Food.size(); ++i )
	{
		m_Food[ i ].draw();
	}
	for( int i= 0; i < m_PacMan.size(); ++i )
	{
		m_PacMan[ i ].draw();
	}
	for( int i= 0; i < m_iPlayerCount; ++i )
	{
		m_Player[ i ].draw();
	}

	if( m_Level > 0 && m_fLevelTime < 3.0f )
	{
		fnt->printf( 
			g_BoardSizeX, 
			g_BoardSizeY, 
			HGETEXT_CENTER,
			"Next Level!\nBonus: %d Points",
			m_Level * 10000 );
	}

	if( m_GameOver )
	{
		fnt->printf( 
			g_BoardSizeX, 
			g_BoardSizeY, 
			HGETEXT_CENTER,
			"Game Over!\nRestart with R or SPACE");
	}
	else if( m_fGameTime < 3.0f )
	{
		fnt->printf( 
			g_BoardSizeX, 
			40.0f, 
			HGETEXT_CENTER,
			"F1 - one player mode\nF2 - coop mode");

		if( m_iPlayerCount == 1 )
		{
			fnt->printf( 
				g_BoardSizeX, 
				g_BoardSizeY, 
				HGETEXT_CENTER,
				"move with CURSOR\nFocus with SHIFT");
		}
		else
		{
			fnt->printf( 
				g_BoardSizeX + 200.0f, 
				g_BoardSizeY, 
				HGETEXT_CENTER,
				"move with CURSOR\nFocus with Numpad 0");
			fnt->printf( 
				g_BoardSizeX - 200.0f, 
				g_BoardSizeY, 
				HGETEXT_CENTER,
				"move with WASD\nFocus with G");
		}
		fnt->printf( 
			g_BoardSizeX, 
			g_BoardSizeY + 280.0f, 
			HGETEXT_CENTER,
			"code: TGGC\nsoundtrack: Chrome - Stardust");
	}
	fnt->printf( 
		5, 
		5, 
		HGETEXT_LEFT,
		"Score: %07d", 
		m_iScore );
	fnt->printf( 
		g_BoardSizeX * 2 - 5, 
		5, 
		HGETEXT_RIGHT,
		"Level: %02d", 
		GetLevel() + 1 );
	g_hge->Gfx_EndScene();

	return false;
}

int Game::GetLevel()
{
	return m_Level;
}

int Game::GetPlayerCount()
{
	return m_iPlayerCount;
}

//	EOF: Game.cpp

