/*!
	\file PacMan.cpp

	\brief cpp-Datei der Klasse CPacMan
	\author Christian Roesch
	\date 21.6.2010

	Erklaerung	
*/

#include "PacMan.h"
#include "globals.h"
#include "helper.h"

#include <cmath>
#include "resources.h"
#include "Game.h"

/*!
	Der Konstruktor
*/
PacMan::PacMan()
{
	Reset();
}

void PacMan::Reset()
{
	m_fPosX= 0.0f;
	m_fPosY= 0.0f;
	m_fTargetX= 0.0f;
	m_fTargetY= 0.0f;
	m_fAngle= g_hge->Random_Float(-3.14f, 3.14f);
	NewCloneTime();
}

void PacMan::SetPos( float fPosX, float fPosY )
{
	m_fPosX= fPosX;
	m_fPosY= fPosY;
	FindNewTarget();
}

float PacMan::GetX()
{
	return m_fPosX;
}

float PacMan::GetY()
{
	return m_fPosY;
}

void PacMan::clone( PacMan& p )
{
	m_fAngle= p.m_fAngle;
	SetPos( p.m_fPosX, p.m_fPosY );
}

void PacMan::draw()
{
	sprPacMan->RenderEx( 
		m_fPosX + g_BoardSizeX, 
		m_fPosY + g_BoardSizeY,
		m_fAngle,
		g_PacSize / 8.0f);
}

void PacMan::calculate( float fTimeStep )
{
	float Speed= 50.0f + 5.0f * g_Game.GetLevel();
	Speed*= fTimeStep;

	float fDistX= m_fPosX - m_fTargetX;
	float fDistY= m_fPosY - m_fTargetY;
	float fDist= sqrt( fDistX * fDistX + fDistY * fDistY );

	if( fDist < Speed * 5.0f )
	{
		FindNewTarget();
	}
	else
	{
		m_fPosX-= fDistX / fDist * Speed;
		m_fPosY-= fDistY / fDist * Speed;
	}
	m_fCloneTime-= fTimeStep;
}

void PacMan::canclone( std::vector< PacMan >& PacManList )
{
	if( m_fCloneTime < 0 )
	{
		PacMan p;
		p.clone( *this );
		PacManList.push_back( p );
		NewCloneTime();
	}
}

void PacMan::eat()
{
	m_fCloneTime-= 10.0f;
}

void PacMan::FindNewTarget()
{
	float fDist= g_hge->Random_Float(40.0f, 100.0f);
	m_fAngle+= g_hge->Random_Float(-1.57f, 1.57f);
	m_fTargetX= m_fPosX + cos( m_fAngle ) * fDist;
	m_fTargetY= m_fPosY + sin( m_fAngle ) * fDist;
	Clamp( m_fTargetX, -g_BoardSizeX + g_PacSize, g_BoardSizeX - g_PacSize );
	Clamp( m_fTargetY, -g_BoardSizeY + g_PacSize, g_BoardSizeY - g_PacSize );
}

void PacMan::NewCloneTime()
{
	m_fCloneTime= g_hge->Random_Float(30.0f, 60.0f) + g_Game.GetExtraCloneOffset();
}

//	EOF: PacMan.cpp
