/*!
	\file Player.cpp

	\brief cpp-Datei der Klasse CPlayer
	\author Christian Roesch
	\date 21.6.2010

	Erklaerung	
*/

#include "Player.h"
#include "helper.h"
#include "globals.h"

#include <cmath>
#include "resources.h"

/*!
	Der Konstruktor
*/
Player::Player()
{
	Reset();
}

void Player::Reset()
{
	m_fPosX= 0.0f;
	m_fPosY= 0.0f;
	m_fSpeedX= 0.0f;
	m_fSpeedY= 0.0f;

	m_fVulnerable= 1.0f;

	m_bFocus= false;

	m_fEnergySoundTimer= 0.0f;
}

void Player::SetPos( float fPosX, float fPosY )
{
	m_fPosX= fPosX;
	m_fPosY= fPosY;
}

float Player::GetX()
{
	return m_fPosX;
}

float Player::GetY()
{
	return m_fPosY;
}

float Player::GetFoodPower()
{
	if( m_bFocus )
	{
		return 500.0f + m_fVulnerable * 20.0f;
	}
	else
	{
		return 48.0f;
	}
}

void Player::draw()
{
	float fEner= fabs( m_fVulnerable * 255.0f/ 15.0f );
	if( m_fVulnerable > 0.0f )
	{
		fEner*= 5.0f;
	}
	Clamp( fEner, 0.0f, 255.0f );
	sprGhost2->SetColor( (int)(fEner) << 24 | ( m_fVulnerable > 0.0f ? 0x8040 : 0xa05000) );
	sprGhost2->RenderEx( 
		m_fPosX + g_BoardSizeX, 
		m_fPosY + g_BoardSizeY,
		0.0f,
		1.0f );

	float Factor= -m_fVulnerable * 255.0f / 3.0f;
	Clamp( Factor, 0.0f, 255.0f );
	if( m_bFocus )
	{
		Factor= 0.0f;
	}
	DWORD dwColor = (int)(Factor) << 16 | (int)(255-Factor);
	sprGhost->SetColor( 0xFF000000 | dwColor );
	sprGhost->RenderEx( 
		m_fPosX + g_BoardSizeX, 
		m_fPosY + g_BoardSizeY,
		0.0f,
		g_GhostSize / 16.0f);
}

void Player::calculate( float fTimeStep )
{
	float fAcc= 600.0f * fTimeStep;
	float fFriction= pow( 0.0625f, fTimeStep );

	m_bFocus= false;
	if (g_hge->Input_GetKeyState(m_iKeyFocus))
	{
		fAcc*= 0.25f;
		m_bFocus= true;
	}


	if (g_hge->Input_GetKeyState(m_iKeyLeft))
	{	
		m_fSpeedX-= fAcc;
	}
	if (g_hge->Input_GetKeyState(m_iKeyRight))
	{	
		m_fSpeedX+= fAcc;
	}
	if (g_hge->Input_GetKeyState(m_iKeyUp))
	{	
		m_fSpeedY-= fAcc;
	}
	if (g_hge->Input_GetKeyState(m_iKeyDown))
	{	
		m_fSpeedY+= fAcc;
	}

	// Do some movement calculations and collision detection	
	m_fSpeedX*= fFriction;
	m_fSpeedY*= fFriction;
	m_fPosX+= m_fSpeedX * fTimeStep;
	m_fPosY+= m_fSpeedY * fTimeStep;

	if( m_fPosX < -g_BoardSizeX + g_GhostSize || m_fPosX > g_BoardSizeX - g_GhostSize )
	{
		m_fSpeedX*= -1.0f;
	}
	if( m_fPosY < -g_BoardSizeY + g_GhostSize || m_fPosY > g_BoardSizeY - g_GhostSize )
	{
		m_fSpeedY*= -1.0f;
	}
	Clamp( m_fPosX, -g_BoardSizeX + g_GhostSize, g_BoardSizeX - g_GhostSize );
	Clamp( m_fPosY, -g_BoardSizeY + g_GhostSize, g_BoardSizeY - g_GhostSize );

	m_fVulnerable+= fTimeStep;
	Clamp( m_fVulnerable, -15.0f, 30.0f );
	m_fEnergySoundTimer-= fTimeStep;
	if( m_fEnergySoundTimer < 0 && m_bFocus && m_fVulnerable < -0.5f )
	{
		m_fEnergySoundTimer= 0.8f + m_fVulnerable / 40.0f;
		playFX( sndEnergy, GetX(), 0.5f - m_fVulnerable / 8.0f );
	}
}

bool Player::IsVulnerable()
{
	return m_fVulnerable > 0.0f || m_bFocus;
}

void Player::FoodTouch( float fTimeStep )
{
	m_fVulnerable-= fTimeStep / 5.0f;
}

void Player::EatPacMan()
{
	m_fVulnerable-= 0.25f;
}

void Player::SetKeys( int iKeyFocus, int iKeyUp, int iKeyLeft, int iKeyDown, int iKeyRight )
{
	m_iKeyFocus= iKeyFocus;
	m_iKeyUp= iKeyUp;
	m_iKeyLeft= iKeyLeft;
	m_iKeyDown= iKeyDown;
	m_iKeyRight= iKeyRight;
}


//	EOF: Player.cpp
